import java.io.*;
import java.util.*;
import java.text.*;

/** 
  This class implements an EventLog CORBA interface. This class is an
  application that provides remote log services for other programs in
  the application suite.
*/

public class EventLogService extends _EventLogImplBase
{
    private static PrintWriter out = null; // File for logging into
    private static DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss"); // Date format to use with time-stamps
    private static Hashtable dispatcher = new Hashtable(); // Type dispatch table

/**
  This method will be called by the ORB to log a remote message.
  <P>
  Please, note that this method should not be called directly
  (i.e., locally) by any program.

  @param aHost Host name the application is running on
  @param aType Message type
  @param aSource Source name
  @param aMessage Text message
  */

    public synchronized void log(String aHost, String aType, String aSource, String aMessage)
    {
        try
        {
            // Dispatch according to the type
            LogCapable handler = (LogCapable) dispatcher.get(aType);

            if( null != handler )
            {
                handler.log(aHost, aType, aSource, aMessage);
                return;
            }

            // Format a logging string for all types without custom handlers
            String logString = aHost+"|"+dateFormat.format(new Date())+"|"+aSource+"|"+aType+"|"+aMessage;

            // Write the log string to the log file
            out.println(logString);

        }
        catch(Exception e)
        {
            // Swallow all exceptions here
        }
    }

    public EventLogService(String name)
    {
        super(name);
    }

/**
  This method will start the event log service.
  */

    public static void main(String[] args)
    {
        try
        {
            // Create the type dispatch table
            for(int i=0; i<args.length; i++)
            {
                String typeName = args[i].substring(0, args[i].indexOf('='));
                String className = args[i].substring(args[i].indexOf('=')+1);

                LogCapable handler = (LogCapable) Class.forName(className).newInstance();
                dispatcher.put(typeName, handler);
            }

            // Initialize the ORB.
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(args, null);

            // Create the service object.
            EventLogService service = new EventLogService("log");

            // Open the log file
            out = new PrintWriter(new BufferedWriter(new FileWriter("logfile.log")), true);

            // Initialize the BOA.
            com.inprise.vbroker.CORBA.BOA boa = ((com.inprise.vbroker.CORBA.ORB)orb).BOA_init();
            boa.obj_is_ready(service);

            // Wait for incoming requests
            boa.impl_is_ready();
        }
        catch(Throwable e)
        {
            System.err.println("Cannot initialize EventLogService: "+e);
            e.printStackTrace(System.err);
            System.exit(1);
        }
    }
}
