public class Log extends Exception
{
    private static EventLog remoteLog = null;    // A stub for the remote service
    private static String hostName = ""; // Now this machine is called

    static
    {
        try
        {
            hostName = java.net.InetAddress.getLocalHost().getHostName();
        }
        catch(java.net.UnknownHostException e){}
    }

/** 
  Constructs a new Log object fully "compatible" with Exception.
  */
    public Log()
    {
        super();
    }   

/** 
  Constructs a new Log object fully "compatible" with Exception.
  */
    public Log(String message)
    {
        super(message);
    }   

/** 

  Constructs a new Log object.  Since this object inherits from
  Exception, it can be immediately thrown. Actual logging will be done
  by the "log" method which has the same parameters.

  */
    public Log(String eventSource, String messageType, String message)
    {
        super(message);
        
        // Log the message
        Log.log(eventSource, messageType, message);
    }   

/** 
  Logs a message.

  @param eventSource Should be used to supply a name for the
  method that tries to log.

  @param messageType One of the values from the set of string that
  event service can recognize. Messages will be treated differently
  depending on their type.

  @param message A string with a meaningful human-readable
  text that will be logged.
  */

    public static void log(String eventSource, String messageType, String aMessage)
    {
        if( null == remoteLog )
        {
            org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(new String[0], null);
            remoteLog = EventLogHelper.bind(orb);
        }

        try
        {
            synchronized(remoteLog)
            {
                remoteLog.log(hostName, messageType, eventSource, aMessage);
            }
        }
        catch(Throwable e)
        {
            // Swallow everything and do not tell anyone.
            // It's a bad thing to do, but it crash proofs applications
            // in the case EventLog service dies.
        }
    }   
}
