/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.myupdateable;

import java.io.Serializable;
import java.awt.Frame;
import java.awt.Button;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import mc.rmi.Service;
import mc.rmi.ServiceState;
import mc.rmi.UpdateableService;
import mc.rmi.UpdateableBase;
import mc.rmi.Requestor;
import mc.rmi.RemoteState;
import mc.rmi.LocalReference;
import mc.rmi.MCLocalReference;
import mc.rmi.ConfigRuntime;
import mc.rmi.Configurator;
import mc.rmi.ConfiguratorBase;
import mc.rmi.ConfigurationAlert;
import mc.rmi.ConfigurationFailure;

/**
 * The implementation class for the MyUpdateable service. This may only
 * implement one interface, and the name of this class must be the interface
 * name with "Impl" appended, and they must be in the same package.
 */
public class MyUpdateableImpl
extends
	UpdateableBase
implements
	MyUpdateable
{
	public static final int[] VersionNo = { 30 };

	private MyRemoteState remoteState = new MyRemoteState();
	private MyLocalState localState;

	public MyUpdateableImpl(String updateServiceHost, String updateServiceName)
	{
		this.remoteState = new MyRemoteState();
	}

	public MyUpdateableImpl()
	{
	}

	// ------------------------------------------------------------
	// From Service:

	/**
	 * Return the version number of this implementation of the MyApi
	 * service.
	 */
	public int[] getServiceVersionNo()
	{
		return VersionNo;
	}

	/**
	 * Create a requestor to pass to the remote update service. The requestor
	 * is a remote reference and is used as a callback object.
	 * Use this method if you are requesting an update to this service.
	 */
	public Requestor createRequestor()
	{
		return new Requestor()
		{
			private int[] v = VersionNo;
			private Class c = MyApi.class;

			public int[] getServiceVersionNoRequestorHas()
			{
				return v;
			}

			public Class getServiceInterfaceRequestorHas()
			{
				return c;
			}
		};
	}

	/**
	 * Create a requestor to pass to the remote update service. The requestor
	 * is a remote reference and is used as a callback object.
	 * Use this method if you are requesting an update to another service.
	 * Null arguments should be used if the client does not yet have
	 * the service.
	 */
	public Requestor createRequestor(final int[] serviceVersionNoRequestorHas,
		final Class serviceInterfaceRequestorHas)
	{
		return new Requestor()
		{
			public int[] getServiceVersionNoRequestorHas()
			{
				return serviceVersionNoRequestorHas;
			}

			public Class getServiceInterfaceRequestorHas()
			{
				return serviceInterfaceRequestorHas;
			}
		};
	}

	/**
	 * Return a configurator for this object type. Create one if necessary,
	 * but do not allow more than one to exist for this object.
	 */
	public Configurator getConfigurator(ConfigRuntime rt)
	{
		Configurator myUpdateableConfigurator = new ConfiguratorBase()
		{
			public void configure()
			throws
				ConfigurationAlert,
				ConfigurationFailure
			{
				final Frame configFrame = new Frame();
				configFrame.setSize(200, 75);

				Button b = new Button("Local Config.");
				b.addActionListener
				(
					new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							dispose();
							configFrame.dispose();
						}
					}
				);

				configFrame.add(b);

				configFrame.show();

				super.configure();
			}
		};

		myUpdateableConfigurator.setService(this);
		localState = new MyLocalState(rt);

		return myUpdateableConfigurator;
	}

	public void doInit()
	{
	}

	public void doStart()
	{
	}

	public void doStop()
	{
	}

	public void doDeactivate()
	{
	}

	// ------------------------------------------------------------
	// From UpdateableService:

	/**
	 * Clone this obtect's remote state and return it.
	 * The remote state encapsulates the portion of this object's state that
	 * can be configured remotely when this object is updated.
	 */
	public RemoteState copyRemoteState()
	{
		return (RemoteState)(remoteState.clone());
	}

	/**
	 * Create and return a local (managed) reference to this object's local
	 * state. The local state encapsulates the portion of this object's
	 * state that cannot be configured remotely when this object is updated.
	 */
	public LocalReference getLocalState()
	{
		return new MCLocalReference(localState.runtime, localState);
	}

	// ------------------------------------------------------------
	// From MyApi: (none).

	// ------------------------------------------------------------
	// Protected:

	protected static class MyRemoteState implements RemoteState, Cloneable
	{
		public MyRemoteState()
		{
		}

		public Object clone()
		{
			Object o = null;
			try
			{
				o = super.clone();
			}
			catch (CloneNotSupportedException ex)
			{
				ex.printStackTrace();
				throw new RuntimeException("Clone not supported");
			}

			return o;
		}
	}

	protected static class MyLocalState
	{
		public ConfigRuntime runtime;

		public MyLocalState(ConfigRuntime rt)
		{
			this.runtime = rt;
		}
	}
}
