/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.rmi;

public abstract class ConfiguratorBase implements Configurator
{
	/**
	 * Configure a new updateable object based on the local configuration
	 * requirements. This is the persistent configuration. (Runtime
	 * configuration is performed by the init() method.) This assumes
	 * that the object has already been
	 * configured remotely by the updater service that created the new
	 * instance. Here we are primarily concerned with UI-related or
	 * platform-specific configuration - i.e. aspects that the remote
	 * updater service cannot handle.
	 */
	public synchronized void configure()
	throws
	 	ConfigurationAlert,
	 	ConfigurationFailure
	{
		try
		{
			System.out.println("waiting");
			wait();
		}
		catch (InterruptedException ex)
		{
			// For now just continue.
		}

		if (exception == null) return;
		else throw new ConfigurationFailure(exception);
	}

	public void setService(Service s) { this.service = s; }

	public Service getService() { return service; }

	public synchronized void dispose()
	{
		System.out.println("resuming");
		notify();
	}

	protected Service service;
	protected Throwable exception;
}
