/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.rmi;

import java.rmi.Remote;
import java.io.Serializable;
import java.lang.reflect.Proxy;
import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

public class InvocationProxy implements java.lang.reflect.InvocationHandler
{
	private static ThreadLocal threadContext = new ThreadLocal();
	private Remote remote;

	public static Object newInstance(Remote remote)
	{
		return java.lang.reflect.Proxy.newProxyInstance
		(
			remote.getClass().getClassLoader(),
			remote.getClass().getInterfaces(),
			new InvocationProxy(remote)
		);
	}

	public static void setServiceReference(LocalReference sref)
	{
		threadContext.set(sref);
	}

	private InvocationProxy(Object remote)
	{
		this.remote = remote;
	}

	public Object invoke(Object proxy, Method m, Object[] args)
	throws
		Throwable
	{
		Object result;
		try
		{
			//
			// Construct the context object to be passed to the server.

			String svcName;
			int[] version;

			LocalReference sref = (LocalReference)(threadContext.get());
			sychronized (sref)
			{
				Service svc = sref.get(null);
				Class[] interfaces = svc.getClass().getInterfaces();
				svcName = interfaces[0].getName();
				svcVersion = svc.getServiceVersionNo();
			}

			Context context = new Context(svcName, svcVersion);

			//
			// Get information we need about the method and its arguments.
			String name = m.getName();
			Class[] pTypes = m.getParameterTypes();

			//
			// Construct a new argument type array identical to the method,
			// but with the 'Context' type prepended.

			Class[] pTypes2 = Array.newInstance(Class.class, pTypes.length+1);
			pTypes2[0] = Context.class;
			for (int i=0; i < pTypes.length; i++) pTypes2[i+1] = pTypes[i];

			//
			// Find the overloaded version of the method with a Context argument.

			Method m2 = cla.getMethod(name, pTypes2);

			// Copy arguments to a new array, with the context prepended.

			Object[] args2 = Array.newInstance(Object.class, args.length+1);
			args2[0] = context;
			for (i=0; i < args.length; i++) args2[i+1] = args[i];

			//
			// Perform the call, but with the overloaded version of the method.

			result = m2.invoke(remote, args2);
		}
		catch (InvocationTargetException ex)
		{
			throw ex.getTargetException();
		}
		catch (Exception ex2)
		{
			throw new RuntimeException("unexpected invocation exception: " +
				ex2.getMessage());
		}

		return result;
	}
}

class Context implements Serializable
{
	public Context(String name, int[] v)
	{
		this.requestorServiceName = name;
		this.requestorServiceVersion = v;
	}

	public String requestorServiceName;
	public int[] requestorServiceVersion;
}

