/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.rmi;

public interface LocalReference
{
	/**
	 * Reclaim the reference. Users of the reference are expected to
	 * synchronize on this local reference object during the time
	 * they hold a reference to the referent; and to have registered a
	 * listener for notification when reclamation occurs. This method
	 * should notify those listeners.
	 * Note: this should be implemented with a timeout.
	 */
	public void clear()
	throws
		ReferenceAccessTimeout;

	/**
	 * Obtain a reference to the actual (managed) object.
	 * A notification listenener may be registered at this time.
	 * Successive calls to get() may specify the same listener.
	 * However, a listener should only listen for notifications for
	 * a single local reference object.
	 */
	public Object get(NullificationListener listener)
	throws
		ReferentIsNull;

	/**
	 * Check if the referent is null.
	 */
	public boolean isNull();

	/**
	 * Set the value of the reference (ie. the value returned by get()).
	 * If the reference already contains a value, a runtime exception is
	 * thrown. Also, if the runtime times out trying to access this, a
	 * runtime exception is also thrown.
	 */
	public void set(Object value);
}
