/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.rmi;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.RMISecurityManager;
import java.awt.Frame;
import java.awt.Button;
import java.awt.TextField;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.security.AccessController;
import java.security.PrivilegedAction;

public class RemoteUpdaterServant extends UnicastRemoteObject
implements ServiceProvider
{
	private static Frame frame;
	private static TextField tf;
	private static Button b;

	public static void main(String[] args) throws Exception
	{
		System.setSecurityManager(new RMISecurityManager());

		frame = new Frame();
		frame.setLayout(new FlowLayout());
		frame.setSize(550, 75);

		tf = new TextField("", 40);
		frame.add(tf);

		b = new Button("Set java.rmi.server.codebase");
		b.addActionListener
		(
			new ActionListener()
			{

				//....tbd:
				// Note: See if we can dynamically change the codebase
				// if we use a new class loader.

				public void actionPerformed(ActionEvent e)
				{
					AccessController.doPrivileged
					(
						new PrivilegedAction()
						{
							public Object run()
							{
								System.setProperty(
									"java.rmi.server.codebase", tf.getText());
								System.out.println(
									"java.rmi.server.codebase  set to " +
									System.getProperty("java.rmi.server.codebase"));

								return null;
							}
						}
					);
				}
			}
		);

		frame.add(b);

		frame.show();


		RemoteUpdaterServant servant = new RemoteUpdaterServant();
		Registry registry = LocateRegistry.getRegistry();
		System.out.println("Registering " + ServiceName);
		registry.rebind(ServiceName, servant);
	}

	public RemoteUpdaterServant() throws RemoteException
	{
		super();
	}

	public Service getNewInstance
	(
		String svcInterfaceName,
		Requestor requestor
	)
	throws
		CallerIsCurrent,
		NoUpdateAvailable,
	 	ServiceNotFound,
		RemoteException
	{
		//
		// Construct a new (serializable) instance of the updateable object.

		Class svcInterfaceReqHas;
		try
		{
			svcInterfaceReqHas = requestor.getServiceInterfaceRequestorHas();
		}
		catch (RuntimeException ex)
		{
			ex.printStackTrace();
			throw ex;
		}

		//....Note: Check (prove) what happens when svcInterfaceReqHas is not
		//identical to svcInterface.
		Class svcClass;
		try
		{
			svcClass = Class.forName(svcInterfaceName + "Impl");
		}
		catch (ClassNotFoundException ex)
		{
			ex.printStackTrace();
			throw new ServiceNotFound(ex);
		}

		Service s;
		try
		{
			s = (Service)(svcClass.newInstance());
		}
		catch (InstantiationException ex)
		{
			ex.printStackTrace();
			throw new ServiceNotFound(ex);
		}
		catch (IllegalAccessException ex2)
		{
			ex2.printStackTrace();
			throw new RuntimeException(ex2.getMessage());
		}
		catch (RuntimeException ex3)
		{
			ex3.printStackTrace();
			throw ex3;
		}

		//
		// Determine if requestor has this version.

		int[] reqHasVersion = requestor.getServiceVersionNoRequestorHas();
		int[] newVersion = s.getServiceVersionNo();

		try
		{
			if (sameVersion(reqHasVersion, newVersion))
			{
				System.out.println("Caller is current: Version=" + reqHasVersion[0]);
				throw new CallerIsCurrent();
			}
		}
		catch (RuntimeException ex)
		{
			ex.printStackTrace();
			throw ex;
		}

		//
		// Now configure the remote state.
		// This should include a full elaboration of the object tree
		// to ensure that correct codebase annotations are made for all
		// required classes.

		//....tbd

		System.out.println("Returning version " + newVersion[0]);

		return s;
	}

	protected boolean sameVersion(int[] v1, int[] v2)
	{
		if ((v1 == null) || (v2 == null)) return false;
		if (v1.length != v2.length) return false;
		for (int i = 0; i < v1.length; i++) if (v1[i] != v2[i]) return false;
		return true;
	}
}
