/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.rmi;

import java.io.Serializable;

public interface Service extends Serializable
{
	/**
	 * Return the version number of this implementation of the
	 * service.
	 */
	public int[] getServiceVersionNo();

	/**
	 * Create a requestor to pass to the remote update service. The requestor
	 * is a remote reference and is used as a callback object.
	 * Use this method if you are requesting an update to this service.
	 * Applications do not call this method: it is called by the runtime.
	 */
	public Requestor createRequestor();

	/**
	 * Create a requestor to pass to the remote update service. The requestor
	 * is a remote reference and is used as a callback object.
	 * Use this method if you are requesting an update to another service.
	 * Null arguments should be used if the client does not yet have
	 * the service.
	 * Applications do not call this method: it is called by the runtime.
	 */
	public Requestor createRequestor(int[] serviceVersionNoRequestorHas,
		Class serviceInterfaceRequestorHas);

	/**
	 * Return a configurator for this object type. Create one if necessary,
	 * but do not allow more than one to exist for this object.
	 * Applications do not call this method: it is called by the runtime.
	 * The service's configurator performs any persistent configuration.
	 */
	public Configurator getConfigurator(ConfigRuntime rt);

	/**
	 * This method is reponsible for loading any persistent configuration.
	 * This is called after the service is instantiated or reinstantiated.
	 * Applications do not call this method: it is called by the runtime.
	 * This method should not block or take a significant amount of
	 * time (more than a few seconds). If it needs a thread, it should
	 * obtain one from the runtime. The service must be configured
	 * before this method is called.
	 */
	public void init();

	/**
	 * Start or restart the service. The service must be in an initialized
	 * state. This method should not block or use the calling thread to
	 * run the service. If it needs a thread or any other runtime
	 * resources, it should obtain them from the runtime.
	 */
	public void start();

	/**
	 * Shut down the service. This is called when the runtime shuts down.
	 * Calling this should not produce an exception. It should be ignored
	 * if the service is not started. The service should release any
	 * runtime resources it has acquired, but retain any state that was
	 * established by the init() method.
	 */
	public void stop();

	/**
	 * Put the service in an inactive state, and release all
	 * resources that it holds. this object. Remove all strong references
	 * to it and all strong references from it.
	 * Release any resources obtained by the init() method.
	 * The stop() method is called first if necessary to release resources
	 * acquired by the start() method or by the running service.
	 * Calling this should not produce an exception.
	 * This method does not uninstall the service.
	 */
	public void deactivate();

	/**
	 * Return the state of this service.
	 */
	public ServiceState getState();
}
