/*
 * Copyright (c) 2001 by Cliff Berg.
 * This code may be used and distributed for research for purposes.
 * No warrantee of any kind is given or implied.
 */

package mc.rmi;

public abstract class ServiceBase implements Service
{
	private ServiceState state = ServiceState.Uninitialized;

	public synchronized void init()
	{
		doInit();

		state = ServiceState.Initialized;
	}

	public synchronized void start()
	{
		if (state.isStarted()) return;
		if (! state.isInitializedOrStarted()) init();

		doStart();

		state = ServiceState.Started;
	}

	public synchronized void stop()
	{
		if (! state.isStarted()) return;

		doStop();

		state = ServiceState.Initialized;
	}

	public synchronized void deactivate()
	{
		doDeactivate();

		state = ServiceState.Uninitialized;
	}

	/**
	 * Return the state of this service.
	 */
	public synchronized ServiceState getState()
	{
		return state;
	}

	//
	// Normally, a service that extends this class would only
	// override the following methods. These methods should not
	// block or take more than a few seconds to execute. If an
	// execution thread is needed beyond that, one should be
	// obtained from the runtime.
	//

	public void doInit() {}
	public void doStart() {}
	public void doStop() {}
	public void doDeactivate() {}
}
